import { InstrumentClass } from './chunk-4ZUSEHLH.js';
import { delay } from './chunk-KNPBNSJ7.js';
import { MastraBase } from './chunk-G4MCO7XF.js';
import { RegisteredLogger, LogLevel } from './chunk-ICMEXHKD.js';
import { executeHook, AvailableHooks } from './chunk-HBTQNIAX.js';
import { __name, __privateAdd, __privateSet, __privateGet, __publicField } from './chunk-AJJZUHB4.js';
import { randomUUID } from 'crypto';
import { z } from 'zod';
import { generateText, jsonSchema, generateObject, streamText, streamObject } from 'ai';

// src/llm/model/base.ts
var _mastra, _model;
var _MastraLLMBase = class _MastraLLMBase extends MastraBase {
  constructor({ name, model }) {
    super({
      component: RegisteredLogger.LLM,
      name
    });
    // @ts-ignore
    __privateAdd(this, _mastra);
    __privateAdd(this, _model);
    __privateSet(this, _model, model);
  }
  getProvider() {
    return __privateGet(this, _model).provider;
  }
  getModelId() {
    return __privateGet(this, _model).modelId;
  }
  convertToMessages(messages) {
    if (Array.isArray(messages)) {
      return messages.map((m) => {
        if (typeof m === "string") {
          return {
            role: "user",
            content: m
          };
        }
        return m;
      });
    }
    return [
      {
        role: "user",
        content: messages
      }
    ];
  }
  __registerPrimitives(p) {
    if (p.telemetry) {
      this.__setTelemetry(p.telemetry);
    }
    if (p.logger) {
      this.__setLogger(p.logger);
    }
    __privateSet(this, _mastra, p);
  }
  async __text(input) {
    this.logger.debug(`[LLMs:${this.name}] Generating text.`, {
      input
    });
    throw new Error("Method not implemented.");
  }
  async __textObject(input) {
    this.logger.debug(`[LLMs:${this.name}] Generating object.`, {
      input
    });
    throw new Error("Method not implemented.");
  }
  async generate(messages, options = {}) {
    this.logger.debug(`[LLMs:${this.name}] Generating text.`, {
      messages,
      options
    });
    throw new Error("Method not implemented.");
  }
  async __stream(input) {
    this.logger.debug(`[LLMs:${this.name}] Streaming text.`, {
      input
    });
    throw new Error("Method not implemented.");
  }
  async __streamObject(input) {
    this.logger.debug(`[LLMs:${this.name}] Streaming object.`, {
      input
    });
    throw new Error("Method not implemented.");
  }
  async stream(messages, options = {}) {
    this.logger.debug(`[LLMs:${this.name}] Streaming text.`, {
      messages,
      options
    });
    throw new Error("Method not implemented.");
  }
};
_mastra = new WeakMap();
_model = new WeakMap();
__name(_MastraLLMBase, "MastraLLMBase");
var MastraLLMBase = _MastraLLMBase;
var _model2, _mastra2;
var _MastraLLM = class _MastraLLM extends MastraLLMBase {
  constructor({ model, mastra }) {
    super({
      name: "aisdk",
      model
    });
    __privateAdd(this, _model2);
    __privateAdd(this, _mastra2);
    __privateSet(this, _model2, model);
    if (mastra) {
      __privateSet(this, _mastra2, mastra);
      if (mastra.logger) {
        this.__setLogger(mastra.logger);
      }
    }
  }
  __registerPrimitives(p) {
    if (p.telemetry) {
      this.__setTelemetry(p.telemetry);
    }
    if (p.logger) {
      this.__setLogger(p.logger);
    }
    __privateSet(this, _mastra2, p);
  }
  getProvider() {
    return __privateGet(this, _model2).provider;
  }
  getModelId() {
    return __privateGet(this, _model2).modelId;
  }
  convertTools(tools) {
    this.logger.debug("Starting tool conversion for LLM");
    const converted = Object.entries(tools || {}).reduce((memo, value) => {
      const k = value[0];
      const tool = value[1];
      if (tool) {
        memo[k] = {
          description: tool.description,
          parameters: tool.inputSchema,
          execute: /* @__PURE__ */ __name(async (props) => {
            try {
              this.logger.debug("Executing tool", {
                tool: k,
                props
              });
              return tool.execute({
                context: props,
                mastra: __privateGet(this, _mastra2)
              });
            } catch (error) {
              this.logger.error("Error executing tool", {
                tool: k,
                props,
                error
              });
              throw error;
            }
          }, "execute")
        };
      }
      return memo;
    }, {});
    this.logger.debug(`Converted tools for LLM`);
    return converted;
  }
  async __text({ runId, messages, maxSteps, tools, convertedTools, temperature, toolChoice = "auto", onStepFinish }) {
    const model = __privateGet(this, _model2);
    this.logger.debug(`[LLM] - Generating text`, {
      runId,
      messages,
      maxSteps,
      tools: Object.keys(tools || convertedTools || {})
    });
    const finalTools = convertedTools || this.convertTools(tools);
    const argsForExecute = {
      model,
      temperature,
      tools: {
        ...finalTools
      },
      toolChoice,
      maxSteps,
      onStepFinish: /* @__PURE__ */ __name(async (props) => {
        onStepFinish?.(JSON.stringify(props, null, 2));
        this.logger.debug("[LLM] - Step Change:", {
          text: props?.text,
          toolCalls: props?.toolCalls,
          toolResults: props?.toolResults,
          finishReason: props?.finishReason,
          usage: props?.usage,
          runId
        });
        if (props?.response?.headers?.["x-ratelimit-remaining-tokens"] && parseInt(props?.response?.headers?.["x-ratelimit-remaining-tokens"], 10) < 2e3) {
          this.logger.warn("Rate limit approaching, waiting 10 seconds");
          await delay(10 * 1e3);
        }
      }, "onStepFinish")
    };
    return await generateText({
      messages,
      ...argsForExecute,
      experimental_telemetry: this.experimental_telemetry
    });
  }
  async __textObject({ messages, onStepFinish, maxSteps = 5, tools, convertedTools, structuredOutput, runId, temperature, toolChoice = "auto" }) {
    const model = __privateGet(this, _model2);
    this.logger.debug(`[LLM] - Generating a text object`, {
      runId
    });
    const finalTools = convertedTools || this.convertTools(tools);
    const argsForExecute = {
      model,
      temperature,
      tools: {
        ...finalTools
      },
      maxSteps,
      toolChoice,
      onStepFinish: /* @__PURE__ */ __name(async (props) => {
        onStepFinish?.(JSON.stringify(props, null, 2));
        this.logger.debug("[LLM] - Step Change:", {
          text: props?.text,
          toolCalls: props?.toolCalls,
          toolResults: props?.toolResults,
          finishReason: props?.finishReason,
          usage: props?.usage,
          runId
        });
        if (props?.response?.headers?.["x-ratelimit-remaining-tokens"] && parseInt(props?.response?.headers?.["x-ratelimit-remaining-tokens"], 10) < 2e3) {
          this.logger.warn("Rate limit approaching, waiting 10 seconds", {
            runId
          });
          await delay(10 * 1e3);
        }
      }, "onStepFinish")
    };
    let schema;
    let output = "object";
    if (typeof structuredOutput.parse === "function") {
      schema = structuredOutput;
      if (schema instanceof z.ZodArray) {
        output = "array";
        schema = schema._def.type;
      }
    } else {
      schema = jsonSchema(structuredOutput);
    }
    return await generateObject({
      messages,
      ...argsForExecute,
      output,
      schema,
      experimental_telemetry: this.experimental_telemetry
    });
  }
  async __stream({ messages, onStepFinish, onFinish, maxSteps = 5, tools, convertedTools, runId, temperature, toolChoice = "auto" }) {
    const model = __privateGet(this, _model2);
    this.logger.debug(`[LLM] - Streaming text`, {
      runId,
      messages,
      maxSteps,
      tools: Object.keys(tools || convertedTools || {})
    });
    const finalTools = convertedTools || this.convertTools(tools);
    const argsForExecute = {
      model,
      temperature,
      tools: {
        ...finalTools
      },
      maxSteps,
      toolChoice,
      onStepFinish: /* @__PURE__ */ __name(async (props) => {
        onStepFinish?.(JSON.stringify(props, null, 2));
        this.logger.debug("[LLM] - Stream Step Change:", {
          text: props?.text,
          toolCalls: props?.toolCalls,
          toolResults: props?.toolResults,
          finishReason: props?.finishReason,
          usage: props?.usage,
          runId
        });
        if (props?.response?.headers?.["x-ratelimit-remaining-tokens"] && parseInt(props?.response?.headers?.["x-ratelimit-remaining-tokens"], 10) < 2e3) {
          this.logger.warn("Rate limit approaching, waiting 10 seconds", {
            runId
          });
          await delay(10 * 1e3);
        }
      }, "onStepFinish"),
      onFinish: /* @__PURE__ */ __name(async (props) => {
        onFinish?.(JSON.stringify(props, null, 2));
        this.logger.debug("[LLM] - Stream Finished:", {
          text: props?.text,
          toolCalls: props?.toolCalls,
          toolResults: props?.toolResults,
          finishReason: props?.finishReason,
          usage: props?.usage,
          runId
        });
      }, "onFinish")
    };
    return await streamText({
      messages,
      ...argsForExecute,
      experimental_telemetry: this.experimental_telemetry
    });
  }
  async __streamObject({ messages, onStepFinish, onFinish, maxSteps = 5, tools, convertedTools, structuredOutput, runId, temperature, toolChoice = "auto" }) {
    const model = __privateGet(this, _model2);
    this.logger.debug(`[LLM] - Streaming structured output`, {
      runId,
      messages,
      maxSteps,
      tools: Object.keys(tools || convertedTools || {})
    });
    const finalTools = convertedTools || this.convertTools(tools);
    const argsForExecute = {
      model,
      temperature,
      tools: {
        ...finalTools
      },
      maxSteps,
      toolChoice,
      onStepFinish: /* @__PURE__ */ __name(async (props) => {
        onStepFinish?.(JSON.stringify(props, null, 2));
        this.logger.debug("[LLM] - Stream Step Change:", {
          text: props?.text,
          toolCalls: props?.toolCalls,
          toolResults: props?.toolResults,
          finishReason: props?.finishReason,
          usage: props?.usage,
          runId
        });
        if (props?.response?.headers?.["x-ratelimit-remaining-tokens"] && parseInt(props?.response?.headers?.["x-ratelimit-remaining-tokens"], 10) < 2e3) {
          this.logger.warn("Rate limit approaching, waiting 10 seconds", {
            runId
          });
          await delay(10 * 1e3);
        }
      }, "onStepFinish"),
      onFinish: /* @__PURE__ */ __name(async (props) => {
        onFinish?.(JSON.stringify(props, null, 2));
        this.logger.debug("[LLM] - Stream Finished:", {
          text: props?.text,
          toolCalls: props?.toolCalls,
          toolResults: props?.toolResults,
          finishReason: props?.finishReason,
          usage: props?.usage,
          runId
        });
      }, "onFinish")
    };
    let schema;
    let output = "object";
    if (typeof structuredOutput.parse === "function") {
      schema = structuredOutput;
      if (schema instanceof z.ZodArray) {
        output = "array";
        schema = schema._def.type;
      }
    } else {
      schema = jsonSchema(structuredOutput);
    }
    return streamObject({
      messages,
      ...argsForExecute,
      output,
      schema,
      experimental_telemetry: this.experimental_telemetry
    });
  }
  async generate(messages, { maxSteps = 5, onStepFinish, tools, convertedTools, runId, output = "text", temperature } = {}) {
    const msgs = this.convertToMessages(messages);
    if (output === "text") {
      return await this.__text({
        messages: msgs,
        onStepFinish,
        maxSteps,
        tools,
        convertedTools,
        runId,
        temperature
      });
    }
    return await this.__textObject({
      messages: msgs,
      structuredOutput: output,
      onStepFinish,
      maxSteps,
      tools,
      convertedTools,
      runId
    });
  }
  async stream(messages, { maxSteps = 5, onFinish, onStepFinish, tools, convertedTools, runId, output = "text", temperature } = {}) {
    const msgs = this.convertToMessages(messages);
    if (output === "text") {
      return await this.__stream({
        messages: msgs,
        onStepFinish,
        onFinish,
        maxSteps,
        tools,
        convertedTools,
        runId,
        temperature
      });
    }
    return await this.__streamObject({
      messages: msgs,
      structuredOutput: output,
      onStepFinish,
      onFinish,
      maxSteps,
      tools,
      convertedTools,
      runId,
      temperature
    });
  }
};
_model2 = new WeakMap();
_mastra2 = new WeakMap();
__name(_MastraLLM, "MastraLLM");
var MastraLLM = _MastraLLM;

// src/agent/index.ts
function _ts_decorate(decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
}
__name(_ts_decorate, "_ts_decorate");
function _ts_metadata(k, v) {
  if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
}
__name(_ts_metadata, "_ts_metadata");
var _mastra3, _memory;
var _Agent = class _Agent extends MastraBase {
  constructor(config) {
    super({
      component: RegisteredLogger.AGENT
    });
    __publicField(this, "name");
    __publicField(this, "llm");
    __publicField(this, "instructions");
    __publicField(this, "model");
    __privateAdd(this, _mastra3);
    __privateAdd(this, _memory);
    __publicField(this, "tools");
    __publicField(this, "metrics");
    this.name = config.name;
    this.instructions = config.instructions;
    if (!config.model) {
      throw new Error(`LanugageModel is required to create an Agent. Please provider the 'model'.`);
    }
    this.llm = new MastraLLM({
      model: config.model
    });
    this.tools = {};
    this.metrics = {};
    if (config.tools) {
      this.tools = config.tools;
    }
    if (config.mastra) {
      __privateSet(this, _mastra3, config.mastra);
    }
    if (config.metrics) {
      this.metrics = config.metrics;
    }
    if (config.memory) {
      __privateSet(this, _memory, config.memory);
    }
  }
  hasOwnMemory() {
    return Boolean(__privateGet(this, _memory));
  }
  getMemory() {
    return __privateGet(this, _memory) ?? __privateGet(this, _mastra3)?.memory;
  }
  __updateInstructions(newInstructions) {
    this.instructions = newInstructions;
    this.logger.debug(`[Agents:${this.name}] Instructions updated.`, {
      model: this.model,
      name: this.name
    });
  }
  __registerPrimitives(p) {
    if (p.telemetry) {
      this.__setTelemetry(p.telemetry);
    }
    if (p.logger) {
      this.__setLogger(p.logger);
    }
    this.llm.__registerPrimitives(p);
    __privateSet(this, _mastra3, p);
    this.logger.debug(`[Agents:${this.name}] initialized.`, {
      model: this.model,
      name: this.name
    });
  }
  /**
  * Set the concrete tools for the agent
  * @param tools
  */
  __setTools(tools) {
    this.tools = tools;
    this.logger.debug(`[Agents:${this.name}] Tools set for agent ${this.name}`, {
      model: this.model,
      name: this.name
    });
  }
  async generateTitleFromUserMessage({ message }) {
    const { object } = await this.llm.__textObject({
      messages: [
        {
          role: "system",
          content: `

      - you will generate a short title based on the first message a user begins a conversation with
      - ensure it is not more than 80 characters long
      - the title should be a summary of the user's message
      - do not use quotes or colons`
        },
        {
          role: "user",
          content: JSON.stringify(message)
        }
      ],
      structuredOutput: z.object({
        title: z.string()
      })
    });
    return object.title;
  }
  getMostRecentUserMessage(messages) {
    const userMessages = messages.filter((message) => message.role === "user");
    return userMessages.at(-1);
  }
  async genTitle(userMessage) {
    let title = "New Thread";
    try {
      if (userMessage) {
        title = await this.generateTitleFromUserMessage({
          message: userMessage
        });
      }
    } catch (e) {
      console.error("Error generating title:", e);
    }
    return title;
  }
  async saveMemory({ threadId, memoryConfig, resourceId, userMessages, runId }) {
    const userMessage = this.getMostRecentUserMessage(userMessages);
    const memory = this.getMemory();
    if (memory) {
      let thread;
      if (!threadId) {
        this.logger.debug(`No threadId, creating new thread for agent ${this.name}`, {
          runId: runId || this.name
        });
        const title = await this.genTitle(userMessage);
        thread = await memory.createThread({
          threadId,
          resourceId,
          title,
          memoryConfig
        });
      } else {
        thread = await memory.getThreadById({
          threadId
        });
        if (!thread) {
          this.logger.debug(`Thread with id ${threadId} not found, creating new thread for agent ${this.name}`, {
            runId: runId || this.name
          });
          const title = await this.genTitle(userMessage);
          thread = await memory.createThread({
            threadId,
            resourceId,
            title,
            memoryConfig
          });
        }
      }
      const newMessages = userMessage ? [
        userMessage
      ] : userMessages;
      if (thread) {
        const messages = newMessages.map((u) => {
          return {
            id: this.getMemory()?.generateId(),
            createdAt: /* @__PURE__ */ new Date(),
            threadId: thread.id,
            ...u,
            content: u.content,
            role: u.role,
            type: "text"
          };
        });
        const contextCallMessages = [
          {
            role: "system",
            content: `

             Analyze this message to determine if the user is referring to a previous conversation with the LLM.
             Specifically, identify if the user wants to reference specific information from that chat or if they want the LLM to use the previous chat messages as context for the current conversation.
             Extract any date ranges mentioned in the user message that could help identify the previous chat.
             Return dates in ISO format.
             If no specific dates are mentioned but time periods are (like "last week" or "past month"), calculate the appropriate date range.
             For the end date, return the date 1 day after the end of the time period.
             Today's date is ${(/* @__PURE__ */ new Date()).toISOString()}`
          },
          ...newMessages
        ];
        let context;
        try {
          context = await this.llm.__textObject({
            messages: contextCallMessages,
            structuredOutput: z.object({
              usesContext: z.boolean(),
              startDate: z.date(),
              endDate: z.date()
            })
          });
          this.logger.debug("Text Object result", {
            contextObject: JSON.stringify(context.object, null, 2),
            runId: runId || this.name
          });
        } catch (e) {
          if (e instanceof Error) {
            this.log(LogLevel.DEBUG, `No context found: ${e.message}`);
          }
        }
        const memoryMessages = threadId && memory ? (await memory.rememberMessages({
          threadId,
          config: memoryConfig,
          vectorMessageSearch: messages.slice(-1).map((m) => {
            if (typeof m === `string`) {
              return m;
            }
            return m?.content || ``;
          }).join(`
`)
        })).messages : [];
        if (memory) {
          await memory.saveMessages({
            messages,
            memoryConfig
          });
        }
        this.log(LogLevel.DEBUG, "Saved messages to memory", {
          threadId: thread.id,
          runId
        });
        const memorySystemMessage = memory && threadId ? await memory.getSystemMessage({
          threadId,
          memoryConfig
        }) : null;
        return {
          threadId: thread.id,
          messages: [
            {
              role: "system",
              content: `

             Analyze this message to determine if the user is referring to a previous conversation with the LLM.
             Specifically, identify if the user wants to reference specific information from that chat or if they want the LLM to use the previous chat messages as context for the current conversation.
             Extract any date ranges mentioned in the user message that could help identify the previous chat.
             Return dates in ISO format.
             If no specific dates are mentioned but time periods are (like "last week" or "past month"), calculate the appropriate date range.
             For the end date, return the date 1 day after the end of the time period.
             Today's date is ${(/* @__PURE__ */ new Date()).toISOString()} and the time is ${(/* @__PURE__ */ new Date()).toLocaleTimeString("en-US", {
                hour: "numeric",
                minute: "numeric",
                hour12: true
              })} ${memorySystemMessage ? `

${memorySystemMessage}` : ""}`
            },
            ...memoryMessages,
            ...newMessages
          ]
        };
      }
      return {
        threadId: thread?.id || threadId || "",
        messages: userMessages
      };
    }
    return {
      threadId: threadId || "",
      messages: userMessages
    };
  }
  async saveResponse({ result, threadId, resourceId, runId, memoryConfig }) {
    const { response } = result;
    try {
      if (response.messages) {
        const ms = Array.isArray(response.messages) ? response.messages : [
          response.messages
        ];
        const responseMessagesWithoutIncompleteToolCalls = this.sanitizeResponseMessages(ms);
        const memory = this.getMemory();
        if (memory) {
          this.logger.debug(`[Agent:${this.name}] - Memory persistence: store=${this.getMemory()?.constructor.name} threadId=${threadId}`, {
            runId,
            resourceId,
            threadId,
            memoryStore: this.getMemory()?.constructor.name
          });
          await memory.saveMessages({
            memoryConfig,
            messages: responseMessagesWithoutIncompleteToolCalls.map((message) => {
              const messageId = randomUUID();
              let toolCallIds;
              let toolCallArgs;
              let toolNames;
              let type = "text";
              if (message.role === "tool") {
                toolCallIds = message.content.map((content) => content.toolCallId);
                type = "tool-result";
              }
              if (message.role === "assistant") {
                const assistantContent = message.content;
                const assistantToolCalls = assistantContent.map((content) => {
                  if (content.type === "tool-call") {
                    return {
                      toolCallId: content.toolCallId,
                      toolArgs: content.args,
                      toolName: content.toolName
                    };
                  }
                  return void 0;
                })?.filter(Boolean);
                toolCallIds = assistantToolCalls?.map((toolCall) => toolCall.toolCallId);
                toolCallArgs = assistantToolCalls?.map((toolCall) => toolCall.toolArgs);
                toolNames = assistantToolCalls?.map((toolCall) => toolCall.toolName);
                type = assistantContent?.[0]?.type;
              }
              return {
                id: messageId,
                threadId,
                role: message.role,
                content: message.content,
                createdAt: /* @__PURE__ */ new Date(),
                toolCallIds: toolCallIds?.length ? toolCallIds : void 0,
                toolCallArgs: toolCallArgs?.length ? toolCallArgs : void 0,
                toolNames: toolNames?.length ? toolNames : void 0,
                type
              };
            })
          });
        }
      }
    } catch (err) {
      this.logger.error(`[Agent:${this.name}] - Failed to save assistant response`, {
        error: err,
        runId
      });
    }
  }
  sanitizeResponseMessages(messages) {
    let toolResultIds = [];
    for (const message of messages) {
      if (message.role === "tool") {
        for (const content of message.content) {
          if (content.type === "tool-result") {
            toolResultIds.push(content.toolCallId);
          }
        }
      }
    }
    const messagesBySanitizedContent = messages.map((message) => {
      if (message.role !== "assistant") return message;
      if (typeof message.content === "string") return message;
      const sanitizedContent = message.content.filter((content) => content.type === "tool-call" ? toolResultIds.includes(content.toolCallId) : content.type === "text" ? content.text.length > 0 : true);
      return {
        ...message,
        content: sanitizedContent
      };
    });
    return messagesBySanitizedContent.filter((message) => message.content.length > 0);
  }
  convertTools({
    toolsets,
    // threadId,
    runId
  }) {
    this.logger.debug(`[Agents:${this.name}] - Assigning tools`, {
      runId
    });
    const converted = Object.entries(this.tools || {}).reduce((memo, value) => {
      const k = value[0];
      const tool = this.tools[k];
      if (tool) {
        memo[k] = {
          description: tool.description,
          parameters: tool.inputSchema,
          execute: /* @__PURE__ */ __name(async (args) => {
            try {
              this.logger.debug(`[Agent:${this.name}] - Executing tool ${k}`, {
                name: k,
                description: tool.description,
                args
              });
              return tool.execute({
                context: args,
                mastra: __privateGet(this, _mastra3)
              });
            } catch (err) {
              this.logger.error(`[Agent:${this.name}] - Failed execution`, {
                error: err,
                runId
              });
              throw err;
            }
          }, "execute")
        };
      }
      return memo;
    }, {});
    const toolsFromToolsetsConverted = {
      ...converted
    };
    const toolsFromToolsets = Object.values(toolsets || {});
    if (toolsFromToolsets.length > 0) {
      this.logger.debug(`[Agent:${this.name}] - Adding tools from toolsets ${Object.keys(toolsets || {}).join(", ")}`, {
        runId
      });
      toolsFromToolsets.forEach((toolset) => {
        Object.entries(toolset).forEach(([toolName, tool]) => {
          const toolObj = tool;
          toolsFromToolsetsConverted[toolName] = {
            description: toolObj.description || "",
            parameters: toolObj.inputSchema,
            execute: /* @__PURE__ */ __name(async (args) => {
              try {
                this.logger.debug(`[Agent:${this.name}] - Executing tool ${toolName}`, {
                  name: toolName,
                  description: toolObj.description,
                  args
                });
                return toolObj.execute({
                  context: args
                });
              } catch (err) {
                this.logger.error(`[Agent:${this.name}] - Failed toolset execution`, {
                  error: err,
                  runId
                });
                throw err;
              }
            }, "execute")
          };
        });
      });
    }
    return toolsFromToolsetsConverted;
  }
  async preExecute({ resourceId, runId, threadId, memoryConfig, messages }) {
    let coreMessages = [];
    let threadIdToUse = threadId;
    this.log(LogLevel.INFO, `Saving user messages in memory for agent ${this.name}`, {
      runId
    });
    const saveMessageResponse = await this.saveMemory({
      threadId,
      resourceId,
      userMessages: messages,
      memoryConfig
    });
    coreMessages = saveMessageResponse.messages;
    threadIdToUse = saveMessageResponse.threadId;
    return {
      coreMessages,
      threadIdToUse
    };
  }
  __primitive({ messages, context, threadId, memoryConfig, resourceId, runId, toolsets }) {
    return {
      before: /* @__PURE__ */ __name(async () => {
        if (process.env.NODE_ENV !== "test") {
          this.logger.debug(`[Agents:${this.name}] - Starting generation`, {
            runId
          });
        }
        const systemMessage = {
          role: "system",
          content: `${this.instructions}. Today's date is ${(/* @__PURE__ */ new Date()).toISOString()}`
        };
        let coreMessages = messages;
        let threadIdToUse = threadId;
        if (this.getMemory() && resourceId) {
          this.logger.debug(`[Agent:${this.name}] - Memory persistence enabled: store=${this.getMemory()?.constructor.name}, resourceId=${resourceId}`, {
            runId,
            resourceId,
            threadId: threadIdToUse,
            memoryStore: this.getMemory()?.constructor.name
          });
          const preExecuteResult = await this.preExecute({
            resourceId,
            runId,
            threadId: threadIdToUse,
            memoryConfig,
            messages
          });
          coreMessages = preExecuteResult.coreMessages;
          threadIdToUse = preExecuteResult.threadIdToUse;
        }
        let convertedTools;
        if (toolsets && Object.keys(toolsets || {}).length > 0 || this.getMemory() && resourceId) {
          const reasons = [];
          if (toolsets && Object.keys(toolsets || {}).length > 0) {
            reasons.push(`toolsets present (${Object.keys(toolsets || {}).length} tools)`);
          }
          if (this.getMemory() && resourceId) {
            reasons.push("memory and resourceId available");
          }
          this.logger.debug(`[Agent:${this.name}] - Enhancing tools: ${reasons.join(", ")}`, {
            runId,
            toolsets: toolsets ? Object.keys(toolsets) : undefined,
            hasMemory: !!this.getMemory(),
            hasResourceId: !!resourceId
          });
          convertedTools = this.convertTools({
            toolsets,
            threadId: threadIdToUse,
            runId
          });
        }
        const messageObjects = [
          systemMessage,
          ...context || [],
          ...coreMessages
        ];
        return {
          messageObjects,
          convertedTools,
          threadId: threadIdToUse
        };
      }, "before"),
      after: /* @__PURE__ */ __name(async ({ result, threadId: threadId2, memoryConfig: memoryConfig2, outputText, runId: runId2 }) => {
        const resToLog = {
          text: result?.text,
          object: result?.object,
          toolResults: result?.toolResults,
          toolCalls: result?.toolCalls,
          usage: result?.usage,
          steps: result?.steps?.map((s) => {
            return {
              stepType: s?.stepType,
              text: result?.text,
              object: result?.object,
              toolResults: result?.toolResults,
              toolCalls: result?.toolCalls,
              usage: result?.usage
            };
          })
        };
        this.logger.debug(`[Agent:${this.name}] - Post processing LLM response`, {
          runId: runId2,
          result: resToLog,
          threadId: threadId2
        });
        if (this.getMemory() && resourceId) {
          try {
            await this.saveResponse({
              result,
              threadId: threadId2,
              resourceId,
              memoryConfig: memoryConfig2,
              runId: runId2
            });
          } catch (e) {
            this.logger.error("Error saving response", {
              error: e,
              runId: runId2,
              result: resToLog,
              threadId: threadId2
            });
          }
        }
        if (Object.keys(this.metrics || {}).length > 0) {
          const input = messages.map((message) => message.content).join("\n");
          const runIdToUse = runId2 || crypto.randomUUID();
          for (const metric of Object.values(this.metrics || {})) {
            executeHook(AvailableHooks.ON_GENERATION, {
              input,
              output: outputText,
              runId: runIdToUse,
              metric,
              agentName: this.name,
              instructions: this.instructions
            });
          }
        }
      }, "after")
    };
  }
  async generate(messages, { context, threadId: threadIdInFn, memoryOptions, resourceId, maxSteps = 5, onStepFinish, runId, toolsets, output = "text", temperature, toolChoice = "auto" } = {}) {
    let messagesToUse = [];
    if (typeof messages === `string`) {
      messagesToUse = [
        {
          role: "user",
          content: messages
        }
      ];
    } else {
      messagesToUse = messages.map((message) => {
        if (typeof message === `string`) {
          return {
            role: "user",
            content: message
          };
        }
        return message;
      });
    }
    const runIdToUse = runId || randomUUID();
    const { before, after } = this.__primitive({
      messages: messagesToUse,
      context,
      threadId: threadIdInFn,
      memoryConfig: memoryOptions,
      resourceId,
      runId: runIdToUse,
      toolsets
    });
    const { threadId, messageObjects, convertedTools } = await before();
    if (output === "text") {
      const result2 = await this.llm.__text({
        messages: messageObjects,
        tools: this.tools,
        convertedTools,
        onStepFinish,
        maxSteps,
        runId: runIdToUse,
        temperature,
        toolChoice
      });
      const outputText2 = result2.text;
      await after({
        result: result2,
        threadId,
        memoryConfig: memoryOptions,
        outputText: outputText2,
        runId: runIdToUse
      });
      return result2;
    }
    const result = await this.llm.__textObject({
      messages: messageObjects,
      tools: this.tools,
      structuredOutput: output,
      convertedTools,
      onStepFinish,
      maxSteps,
      runId: runIdToUse,
      temperature,
      toolChoice
    });
    const outputText = JSON.stringify(result.object);
    await after({
      result,
      threadId,
      memoryConfig: memoryOptions,
      outputText,
      runId: runIdToUse
    });
    return result;
  }
  async stream(messages, { context, threadId: threadIdInFn, memoryOptions, resourceId, maxSteps = 5, onFinish, onStepFinish, runId, toolsets, output = "text", temperature, toolChoice = "auto" } = {}) {
    const runIdToUse = runId || randomUUID();
    let messagesToUse = [];
    if (typeof messages === `string`) {
      messagesToUse = [
        {
          role: "user",
          content: messages
        }
      ];
    } else {
      messagesToUse = messages.map((message) => {
        if (typeof message === `string`) {
          return {
            role: "user",
            content: message
          };
        }
        return message;
      });
    }
    const { before, after } = this.__primitive({
      messages: messagesToUse,
      context,
      threadId: threadIdInFn,
      memoryConfig: memoryOptions,
      resourceId,
      runId: runIdToUse,
      toolsets
    });
    const { threadId, messageObjects, convertedTools } = await before();
    if (output === "text") {
      this.logger.debug(`Starting agent ${this.name} llm stream call`, {
        runId
      });
      return this.llm.__stream({
        messages: messageObjects,
        temperature,
        tools: this.tools,
        convertedTools,
        onStepFinish,
        onFinish: /* @__PURE__ */ __name(async (result) => {
          try {
            const res = JSON.parse(result) || {};
            const outputText = res.text;
            await after({
              result: res,
              threadId,
              memoryConfig: memoryOptions,
              outputText,
              runId: runIdToUse
            });
          } catch (e) {
            this.logger.error("Error saving memory on finish", {
              error: e,
              runId
            });
          }
          onFinish?.(result);
        }, "onFinish"),
        maxSteps,
        runId,
        toolChoice
      });
    }
    this.logger.debug(`Starting agent ${this.name} llm streamObject call`, {
      runId
    });
    return this.llm.__streamObject({
      messages: messageObjects,
      tools: this.tools,
      temperature,
      structuredOutput: output,
      convertedTools,
      onStepFinish,
      onFinish: /* @__PURE__ */ __name(async (result) => {
        try {
          const res = JSON.parse(result) || {};
          const outputText = JSON.stringify(res.object);
          await after({
            result: res,
            threadId,
            memoryConfig: memoryOptions,
            outputText,
            runId: runIdToUse
          });
        } catch (e) {
          this.logger.error("Error saving memory on finish", {
            error: e,
            runId
          });
        }
        onFinish?.(result);
      }, "onFinish"),
      maxSteps,
      runId,
      toolChoice
    });
  }
};
_mastra3 = new WeakMap();
_memory = new WeakMap();
__name(_Agent, "Agent");
var Agent = _Agent;
Agent = _ts_decorate([
  InstrumentClass({
    prefix: "agent",
    excludeMethods: [
      "__setTools",
      "__setLogger",
      "__setTelemetry",
      "log"
    ]
  }),
  _ts_metadata("design:type", Function),
  _ts_metadata("design:paramtypes", [
    typeof AgentConfig === "undefined" ? Object : AgentConfig
  ])
], Agent);

export { Agent };
